#! /bin/bash

GRUB_PASSWORD_CONFIG=/etc/grub.d/40_custom
GRUB_PASSWORD_CONFIG_BAK=/etc/grub.d/40_custom_bak
CONTENT1_PREFIX='set superusers='
CONTENT2_PREFIX=password_pbkdf2
CONTENT3_PREFIX=grub.pbkdf2
CUSTOM_FILE_PREFIX="#!/bin/sh\n
exec tail -n +3 \$0\n
# This file provides an easy way to add custom menu entries.  Simply type the\n
# menu entries you want to add after this comment.  Be careful not to change\n
# the 'exec tail' line above.\n\n"
TIP_INFO="Please input right arguments"

set -o errexit

# 创建grub菜单用户密码信息
enter_user () {
	if [ -z $1 -o -z $2 ]; then
		echo $TIP_INFO
		exit 1
	fi

	CONTENT1="$CONTENT1_PREFIX\"$1\""
	CONTENT2="$CONTENT2_PREFIX $1"

	# 写配置文件
	echo -e $CUSTOM_FILE_PREFIX > $GRUB_PASSWORD_CONFIG_BAK
	echo -e "$CONTENT1" >> $GRUB_PASSWORD_CONFIG_BAK
	passwd=$(echo -e "$2\n$2\n" | grub-mkpasswd-pbkdf2 |grep PBKDF2 | awk '{print $4}')
	echo -e "$CONTENT2 $passwd" >> $GRUB_PASSWORD_CONFIG_BAK

	if [ $? -ne '0' ]; then
		echo "create config_bak error"
		exit 1
	fi

	# 写配置文件,使用配置替换的方式,而非用sed删除的方式,避免sed过程中出错,配置文件只删除一部分的情况
	chmod +x  $GRUB_PASSWORD_CONFIG_BAK
	mv $GRUB_PASSWORD_CONFIG_BAK $GRUB_PASSWORD_CONFIG
	if [ $? -ne '0' ]; then
		echo "create config error"
		exit 1
	fi

	echo "enter user $1 success"
	exit 0
}

# 移除grub菜单用户密码信息
delete_user () {
	if [ -z $1 ]; then
		echo $TIP_INFO
		exit 1
	fi

	# 检查是否存在该用户
	CONTENT1="$CONTENT1_PREFIX\"$1\""
	CONTENT2="$CONTENT2_PREFIX $1"

	if [ `grep -c "$CONTENT1" $GRUB_PASSWORD_CONFIG` -eq '0' -o `grep -c "$CONTENT1" $GRUB_PASSWORD_CONFIG` -eq '0' ];
	then
		echo "the user $1 was not found"
		exit 1;
	fi

	# 写配置文件
	echo -e $CUSTOM_FILE_PREFIX > $GRUB_PASSWORD_CONFIG_BAK

	if [ $? -ne '0' ]; then
		echo "create config_bak error"
		exit 1
	fi

	# 替换配置文件
	chmod +x  $GRUB_PASSWORD_CONFIG_BAK
	mv $GRUB_PASSWORD_CONFIG_BAK $GRUB_PASSWORD_CONFIG
	if [ $? -ne '0' ]; then
		echo "create config error"
		exit 1
	fi

	echo "delete_user $1 success"
	exit 0
}

if [ "$UID" -ne "0" ]; then
	echo "You must run this as root"
	exit 1
fi

case $1 in
	-u)
		enter_user $2 $3
		;;
	-d)
		delete_user $2
		;;
	*)
		echo $TIP_INFO
		exit 1
esac

